% Computation of exact MFPTs

%% Generating state transition matrix of a Markov chain

% creating bi-directed chain graph on n nodes
n = 21;
I = eye(n, n);
Self = diag(rand(n, 1));
Next = circshift(eye(n, n), -1, 1);
Prev = circshift(eye(n, n), 1, 1);
W = sparse(Self + (I - Self) / 2 * (Next + Prev));
Wt = W';  % for large networks, may want to keep a single instance of the transpose


%% Computing exact MFPTs between all nodes / states

% Example 1: basic computation

M1 = mfpt(W);

% Example 2: computation with pre-computed Perron vector

[pi, ~] = eigs(Wt, 1);  % network W's Perron vector (nodes' eigencentralities) / stationary distribution of the Markov chain
if pi(1) < 0; pi = -pi; end
pi = pi / sum(pi);

M2 = mfpt(W, pi, 'largestreal');

% Example 3: computation with pre-computed transpose of the state transition matrix

M3 = mfpt(W, nan, Wt);

% Example 4: computation with pre-computed W' and supplied parameters for Perron vector computation

toler = 1e-08 * n;
maxiter = 10 * n;
maxtrials = 1;

M4 = mfpt(W, nan, Wt, toler, maxiter, maxtrials);


%% Visualization

plot(digraph(W));
title('State transition matrix');

figure('Position', [10, 10, 1200, 200]);
for i = 1 : 4
	subplot(1, 4, i);
	pcolor(eval(['M', num2str(i)]));
	colormap(1 - autumn);
	colorbar();
	title(sprintf('MFPTs (Example %d)', i));
end