function [ adj ] = gen_network(params)
%GEN_NETWORK

min_nz_val = 0.01;
validate_network = true;

if nargin < 1
	error('Params are not provided.');
end

unstruct_params;

n = nnodes;

if strcmp('er', net_type)
	% custom params: pr_edge
	adj_conn = reshape(randsample([0, 1], n * n, true, [1 - pr_edge, pr_edge]), n, n);
	adj_conn = adj_conn - diag(diag(adj_conn)) + eye(n, n);
	adj_vals = max(min_nz_val, rand(n, n));
	adj = sparse(rowstoch(adj_conn .* adj_vals));
elseif strcmp('clique', net_type)
	adj = sparse(ones(n, n) ./ n);
elseif strcmp('sf', net_type)
	% custom params: sf_gamma
	adj = sparse([ 1 0; 0 1 ]);
	while graphconncomp(adj) > 1
		adj = cnp_gen_scalefree_net(n, sf_gamma);
		directed = true;
		strongly = true;
		adj = get_largest_conncomp(adj, directed, strongly);
		adj = adj - diag(diag(adj)) + eye(size(adj, 1)); 
		[r, c, v] = find(adj);
		v = rand(length(v), 1);
		adj = sparse(r, c, v, size(adj, 1), size(adj, 1));
		% adj = adj';
	end
elseif strcmp('ba', net_type)
	seed_size = round(min(n - 1, max(4, n / 2)));
	seed = gen_network(struct('net_type', 'er', 'nnodes', seed_size, 'pr_edge', 0.6));
	adj = sparse(SFNG(nnodes, mlinks, seed));
	adj = get_largest_conncomp(adj, true, true);
	adj = adj - diag(diag(adj)) + eye(size(adj, 1)); 
	[r, c, v] = find(adj);
	v = rand(length(v), 1);
	adj = sparse(r, c, v, size(adj, 1), size(adj, 1));
elseif strcmp('ws', net_type)
	error('TODO');
elseif strcmp('karate', net_type)
	adj = load_graph_karate();
	adj = double(adj + adj' > 0) + eye(size(adj, 1), size(adj, 1));
	[r, c, v] = find(adj);
	v = rand(length(v), 1);
	adj = sparse(r, c, v, size(adj, 1), size(adj, 1));
elseif strcmp('bi-chain', net_type)
	error('TODO');
elseif strcmp('uni-ring', net_type)
	band = max(0.01, rand(n, n));
	adj = spdiags(band, [0, +1], n, n);
	adj(n, 1) = max(0.01, rand());
	adj = rowstoch(adj);
elseif strcmp('bi-ring', net_type)
	band = max(0.01, rand(n, n));
	adj = spdiags(band, [-1, 0, +1], n, n);
	adj(1, n) = max(0.01, rand());
	adj(n, 1) = max(0.01, rand());
	adj = rowstoch(adj);
elseif strcmp('bi-star', net_type)
	adj = zeros(n, n);
	for i = 1 : n
		adj(1, i) = max(0.01, rand());
		adj(i, i) = max(0.01, rand());
		adj(i, 1) = max(0.01, rand());
	end
	adj = rowstoch(sparse(adj));
else
	error('Unsupported network.');
end

adj = double(adj > 0);

if validate_network
	if ~issparse(adj)
		error('Adjacency matrix is not sparse.');
	end
	if graphconncomp(adj) > 1
		error('The network is disconnected.');
	end
end

end