function [ lcc_adjmat, lcc_ind, ccnum, ccidx ] = get_largest_conncomp(adjmat, directed, strongly)
%GET_LARGEST_CONNCOMP Extracts the largest connected component from a graph
%
%  Input:
%
%   adjmat -- adjacency matrix of the graph
%   directed -- a logical value indicating whether the graph is directed (default: true)
%   strongly -- a logical value indicating whether to look for strongly
%               connected components
%
%  Output:
%
%   lcc_adjmat -- adjmat of the largest connected component
%   lcc_ind -- index of the largest connected component in the original graph
%   ccnum -- number of connected components in the original graph
%   ccidx -- indices of the connected components for each node of the original graph
%

if(nargin < 1)
    error('Adjmat not specified.');
end
if(~ismatrix(adjmat) || size(adjmat, 1) ~= size(adjmat, 2))
    error('Invalid adjmat.');
end

if(nargin < 2)
    directed = true;
end
if(~isscalar(directed))
    error('Invalid directed. A logical scalar is expected.');
end
directed = logical(directed);

if(nargin < 3)
    strongly = false;
end
if(~isscalar(strongly))
    error('Invalid strongly. A logical scalar is expected.');
end
strongly = logical(strongly);

if(~directed)
    strongly = false;
end

[ ccnum, ccidx ] = conncomp(adjmat, directed, ~strongly);

lcc_ind = mode(ccidx);
llc_indicator = (ccidx == lcc_ind);
if(nnz(llc_indicator) == 1)
    warning('The largest connected component contains a single node.');
end
lcc_adjmat = adjmat(llc_indicator, llc_indicator);

end

