function save_open_figs(dest_dir_path, fig_name_prefix, readme_text, use_fignames, use_figtitles, use_prefix, use_ts, copyaspng)
%SAVE__OPEN_FIGS Saves all open figures as .fig and/or .pdf files to directory dest_dir_path
% and, optionally, creates a README file with readme_text in it

ensure_dir_exists_and_writable(dest_dir_path);

tsnow = datestr(clock, 'YYYY-mm-dd_hh-MM-ss');

if(nargin < 2 || isempty(fig_name_prefix))
    fig_name_prefix = 'fig';
end

if(nargin < 3 || isempty(readme_text))
	readme_text = '';
end

if(nargin < 4 || ~islogical(use_fignames))
	use_fignames = false;
end

if(nargin < 5 || ~islogical(use_figtitles))
	use_figtitles = true;
end

if(nargin < 6 || ~islogical(use_prefix))
	use_prefix = true;
end

if(nargin < 7 || ~islogical(use_ts))
	use_ts = true;
end

if(nargin < 8 || ~islogical(copyaspng))
	copyaspng = false;
end

hs = get(0,'children');
for i = 1 : length(hs)
    h = hs(i);
	
	fname = '';
	if(use_ts)
		fname = [fname, '_ts', tsnow];
	end
	if(use_prefix)
		fname = [fname, '_', fig_name_prefix, num2str(i)];
	end
	if(use_fignames)
		fname = [fname, '_', armor_fname(lower(get(h, 'Name')))];
	end
	if(use_figtitles)
		haxes = findall(h, 'type', 'axes');
		for j = 1 : length(haxes)
			htitle = get(haxes(j), 'Title');
			if(~isempty(htitle))
				title = get(htitle, 'String');
				title = armor_field_name(title);
				if(title)
					fname = strcat(fname, '_', title);
					break;
				end
			end
		end
	end

    full_fname = fullfile(dest_dir_path, fname);
    % pdfcrop(h);
    
	try
		save_compact_fig = true;
		savefig(h, sprintf('%s.fig', full_fname), save_compact_fig);
	catch
		try
			saveas(h, sprintf('%s.fig', full_fname), 'fig');
		catch
			fprintf(2, 'Unable to save figure "%s" as FIG.\n', fname);
		end
	end
	
	dpi = 300;
	
	try
		cmd = 'save2pdf(sprintf(''%s.pdf'', full_fname), h, dpi, copyaspng)';
		evalc(cmd);
		% save2pdf(sprintf('%s.pdf', full_fname), h, dpi, copyaspng);
	catch
		try
			saveas(h, sprintf('%s.pdf', full_fname), 'pdf');
		catch
			fprintf(2, 'Unable to save figure "%s" as PDF.\n', fname);
		end
	end
end

if(nargin >= 3 && ~isempty(readme_text))
    readme_fpath = fullfile(dest_dir_path, strcat('ts', tsnow, '_README.txt'));
    f = fopen(readme_fpath, 'w');
    if(f < 0)
        error('Unable to open/create readme file "%s" for writing.');
    end
    fcloser = onCleanup(@() fclose(f));
    fprintf(f, readme_text);
end

end