#ifndef __MEX_SAFE_H__
#define __MEX_SAFE_H__


#include <mutex>
#include <stdarg.h>
#include <cstring>
#include "mex.h"
#include "matrix.h"


static std::mutex mex_mutex;
static std::mutex std_mutex;


inline int mexPrintf_safe(const char *message, ...) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	va_list valist;
	va_start(valist, message);
	int result = mexPrintf(message, valist);
	va_end(valist);
	return result;
}


inline void *memcpy_safe(void *dst, const void *src, size_t num) {
	std::lock_guard<std::mutex> guard(std_mutex);
	return memcpy(dst, src, num);
}


inline int mexCallMATLAB_safe(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[], const char *functionName) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mexCallMATLAB(nlhs, plhs, nrhs, prhs, functionName);
}


inline mwIndex *mxGetIr_safe(const mxArray *pm) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxGetIr(pm);
}


inline mwIndex *mxGetJc_safe(const mxArray *pm) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxGetJc(pm);
}


inline double *mxGetPr_safe(const mxArray *pm) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxGetPr(pm);
}


inline void mexErrMsgTxt_safe(const char *errormsg) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	mexErrMsgTxt(errormsg);
}


inline void *mxMalloc_safe(mwSize size) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxMalloc(size);
}


inline void *mxCalloc_safe(mwSize n, mwSize size) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxCalloc(n, size);
}


inline mxArray *mxCreateSparse_safe(mwSize m, mwSize n, mwSize nzmax, mxComplexity cf) {
	std::lock_guard<std::mutex> guard(mex_mutex);
	return mxCreateSparse(m, n, nzmax, cf);
}


#endif // __MEX_SAFE_H__